/*! \file example.c 

  \author Paolo Ferragina, Dipartimento di Informatica, Universita' di Pisa (Italy).

  \date March 2003.

  \brief An example of use of the library HuffwordLib.a; here the file
  "example.dat" is first compressed and then decompressed. The result
  is compared to the input file in order to check if any error
  occurred. Then the decompression of a piece of the original text is
  performed.
*/




// Include the Library
#include "HuffwordLib.h"


int main()
{

  FILE *textfile;
  struct stat sbuf;
  char *text,*cbody,*cstring,*dstring,*dblock;
  int tlen,clen,dlen,dblocklen,cbodylen,dbytes,tmp;
  Console console;

  textfile = fopen("example.dat","r");
  stat("example.dat",&sbuf);
  tlen = (int)sbuf.st_size;
  text = (char *) mmap(NULL,tlen,PROT_READ,MAP_SHARED,fileno(textfile),0);

  // Compresses the text file, and computes jumper pointers with
  // average distance about 10. Uses the verbose mode (1).
  Huffw_compress(text,tlen,10,&cstring,&clen,1);

  // Decompresses the string s, and initializes the console c. 
  // Uses the verbose mode (1).
  Huffw_decompress(cstring,clen,&dstring,&dlen,0);

  // Print some debugging infos
  printf("Comparing the compressed and decompressed files = %d (0 is OK)\n",
	 strncmp(dstring,text,dlen));



  /* -------------------------------------------------------------- */
  // ---------- A test of decompression and random access ---------
  /* -------------------------------------------------------------- */

  printf("\n---- Some test ----\n");

  // Initialize all the data structures: Dictionary, Hash Table, Canonical Info and Jumpers (if any).
  // Note cbody is the pointer to the compressed body, and cbodylen is its length.
  HDecompress_getBodyAndConsole(cstring,clen,&dlen,&cbody,&cbodylen,&console);

  // Decompresses the first 10 words, d = #bytes decompressed
  HDecompress_nextBlock_tokens(cbody,cbodylen,10,&dblock,&dblocklen,
				   &dbytes,&console); 

  // Prints some debugging infos
  printf("\n10 words from the beginning:\n\"");
  HPrint_string(dblock,dblocklen);
  printf("\"\n");

  // Decompresses the 5 words which precede the byte reached by the previous Decompress
  HDecompress_previousBlock_tokens(cbody+dbytes,dbytes,5,&dblock,
				       &dblocklen,&tmp,&console); 

  // Prints some debugging infos
  printf("\nLast 5 words of the previous phrase:\n\"");
  HPrint_string(dblock,dblocklen);
  printf("\"\n");

  printf("\nDo you wish to see the dictionary content ? (y/n) ");
  if (getchar() == 'y')
    HDictionary_print(&(console.dictionary),&(console.hashtable),1);


  return 0;
}
